@@ -22,6 +22,7 @@
 
 #include <glib.h>
 #include <string.h>
+
 #include "nautilus-burn-drive.h"
 
 static void
@@ -47,6 +48,9 @@ list_drives (void)
 		gint64      size;
 
 		drive = l->data;
+
+		nautilus_burn_drive_unmount (drive);
+
 		type_str = nautilus_burn_drive_get_supported_media_string (drive, FALSE);
 		media_type = nautilus_burn_drive_get_media_type_full (drive,
 								      &is_rewritable,
@@ -58,7 +62,7 @@ list_drives (void)
 		door_is_open = nautilus_burn_drive_door_is_open (drive);
 		display_name = nautilus_burn_drive_get_name_for_display (drive);
 		max_speed_read = nautilus_burn_drive_get_max_speed_read (drive);
-		max_speed_write = nautilus_burn_drive_get_max_speed_read (drive);
+		max_speed_write = nautilus_burn_drive_get_max_speed_write (drive);
 
 		g_print ("name: %s device: %s max_speed_read: %d\n",
 			 display_name,
@@ -107,6 +111,8 @@ list_drives (void)
 int main (int argc, char **argv)
 {
 	g_type_init ();
+	g_thread_init (NULL);
+
 	list_drives ();
 
 	return 0;
