@@ -186,6 +186,22 @@ close_ioctl_handle (gpointer handle)
 
 #endif
 
+static gboolean
+use_gnome_mount (void)
+{
+#ifndef USE_HAL
+	return FALSE;
+#endif
+
+	char *tmp = g_find_program_in_path ("gnome-mount");
+	if (tmp != NULL) {
+		g_free (tmp);
+		return TRUE;
+	} else {
+		return FALSE;
+	}
+}
+
 #ifdef USE_HAL
 static LibHalContext *
 get_hal_context (void)
@@ -1241,22 +1257,15 @@ typedef struct {
 	gboolean    unmount_ok;
 	guint       timeout_tag;
 	GMainLoop  *loop;
-	char       *device;
-	const char *command;
+	GPtrArray  *argv;
 } UnmountData;
 
-static const char *umount_known_locations [] = {
-	"/sbin/umount",
-	"/bin/umount",
-	"/usr/sbin/umount",
-	"/usr/bin/umount",
-	NULL
-};
-
 static void
 free_unmount_data (UnmountData *unmount_data)
 {
-	g_free (unmount_data->device);
+	g_ptr_array_foreach (unmount_data->argv, (GFunc)g_free, NULL);
+	g_ptr_array_free (unmount_data->argv, TRUE);
+
 	g_free (unmount_data);
 }
 
@@ -1304,17 +1313,54 @@ unmount_timeout (gpointer data)
 	return FALSE;
 }
 
-/* Returns the full path to the queried command */
-static const char *
-find_command (const char **known_locations)
-{
-	int i;
+static const char *umount_known_locations [] = {
+	"/sbin/umount",
+	"/bin/umount",
+	"/usr/sbin/umount",
+	"/usr/bin/umount",
+	NULL
+};
 
-	for (i = 0; known_locations [i]; i++){
-		if (g_file_test (known_locations [i], G_FILE_TEST_EXISTS))
-			return known_locations [i];
+/* Returns the full command */
+static GPtrArray *
+create_command (const char *device)
+{
+	GPtrArray *argv;
+
+	argv = g_ptr_array_new ();
+
+	if (use_gnome_mount ()) {
+		char *str;
+
+		str = g_strdup ("gnome-mount");
+		g_ptr_array_add (argv, str);
+		str = g_strdup_printf ("--device=%s", device);
+		g_ptr_array_add (argv, str);
+		str = g_strdup ("--unmount");
+		g_ptr_array_add (argv, str);
+	} else {
+		char *str;
+		int   i;
+
+		str = NULL;
+		for (i = 0; umount_known_locations [i]; i++){
+			if (g_file_test (umount_known_locations [i], G_FILE_TEST_EXISTS)) {
+				str = g_strdup (umount_known_locations [i]);
+				break;
+			}
+		}
+
+		/* no path then try just command */
+		if (str == NULL) {
+			str = g_strdup ("umount");
+		}
+
+		g_ptr_array_add (argv, str);
+		str = g_strdup_printf ("%s", device);
+		g_ptr_array_add (argv, str);
 	}
-	return NULL;
+
+	return argv;
 }
 
 static void *
@@ -1322,8 +1368,6 @@ unmount_thread_start (void *arg)
 {
 	UnmountData *data;
 	gint         exit_status;
-	char        *argv [5];
-	int          i;
 	char        *envp [] = {
 		"LC_ALL=C",
 		NULL
@@ -1333,13 +1377,8 @@ unmount_thread_start (void *arg)
 
 	data->unmount_ok = TRUE;
 	
-	i = 0;
-	argv [i++] = (char *)data->command;
-	argv [i++] = data->device;
-	argv [i++] = NULL;
-	
 	if (g_spawn_sync (NULL,
-			  argv,
+			  (char **)data->argv->pdata,
 			  envp,
 			  G_SPAWN_STDOUT_TO_DEV_NULL | G_SPAWN_STDERR_TO_DEV_NULL,
 			  NULL, NULL,
@@ -1396,8 +1435,8 @@ nautilus_burn_drive_unmount (NautilusBurnDrive *drive)
 	data->timeout_tag = g_timeout_add (5 * 1000,
 					   unmount_timeout,
 					   data);
-	data->command = find_command (umount_known_locations);
-	data->device = g_strdup (drive->device);
+	data->argv = create_command (drive->device);
+
 	g_thread_create (unmount_thread_start, data, FALSE, NULL);
 	
 	GDK_THREADS_LEAVE ();
@@ -1438,7 +1477,12 @@ nautilus_burn_drive_eject (NautilusBurnDrive *drive)
 	if (drive->device == NULL)
 		return FALSE;
 	
-	cmd = g_strdup_printf ("eject %s", drive->device);
+	if (use_gnome_mount ()) {
+		cmd = g_strdup_printf ("gnome-mount --eject --device=%s", drive->device);
+	} else {
+		cmd = g_strdup_printf ("eject %s", drive->device);
+	}
+
 	res = g_spawn_command_line_sync (cmd, NULL, NULL, NULL, NULL);
 	g_free (cmd);
 
